/*
Very simple GLUT program using OSL libraries to
demonstrate use of GL_ARB_shading_language_100.

Orion Sky Lawlor, olawlor@acm.org, 2005/2/10 (Public Domain)
*/
#include "ogl/ext.h"
#include "ogl/main.h"
#include "osl/osl.h" /* timer routines */
#include "osl/viewpoint.h" /* 3D viewpoint utility routines */
#include <GL/glut.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

/* Include library implementations here for fast linking */
#include "ogl/util.cpp"
#include "ogl/fast_mipmaps.c"
#include "ogl/ext.cpp"
#include "ogl/glew.c"
#include "ogl/main.cpp"
#include "ogl/readback.cpp"
#include "ogl/program.cpp"
#include "osl/osl.cpp"
#include "osl/viewpoint.cpp"
void osl::bad(const char *a,const char *b,const char *c,int d) 
{ printf(a,b,c,d); exit(1);}

#include "frag_bench.cpp"

/************ Main Drawing Function **************/

/**
  Program to run on each vertex or pixel fragment.
*/
class oglShaderObject {
	int target;// Program target, like GL_FRAGMENT_SHADER_ARB
	GLhandleARB progName; // OpenGL handle to program
public:
	oglShaderObject(int target_) :target(target_) 
		{progName=glCreateShaderObjectARB(target);}
	
	/// Get our handle (e.g., for glAttachObjectARB)
	GLhandleARB get(void) const {return progName;}
	
	// Replace the current program by the one in this file.
	//  Prints out an error message if the program can't be parsed.
	void read(const char *fName);
	
	// Replace the current program by this one.
	//  Prints out an error message if the program can't be parsed.
	void set(const char *programText);
	
	~oglShaderObject() {glDeleteObjectARB(progName);}
};

void oglShaderObject::read(const char *fName) {
	
	// Read program from file
	int len=65536;
	char *s=new char[len];
	FILE *f=fopen(fName,"r");
	len=fread(s,1,len,f); // reset len to file length
	s[len]=0; // null terminate string
	fclose(f);
	
	// Upload program to graphics card
	set(s);
	
	delete[] s;
}

void checkOp(GLhandleARB obj,int what,const char *where) {
	GLint compiled;
	glGetObjectParameterivARB(obj,what,&compiled);
	if (!compiled) {
		printf("Compile error on program: %s\n",where);
		enum {logSize=1000};
		char log[logSize]; int len=0;
		glGetInfoLogARB(obj, logSize,&len,log);
		printf("Log: \n%s\n",log);
		exit(1);
	}
}
	
void oglShaderObject::set(const char *programText) {
	glShaderSourceARB(progName,1, &programText,NULL);
	glCompileShaderARB(progName);
	checkOp(progName,GL_OBJECT_COMPILE_STATUS_ARB,programText);
	oglCheck(programText);
}


class fragBench : public oglDrawable {
public:
	virtual void draw(const oglViewpoint &vp);
};
void fragBench::draw(const oglViewpoint &vp)
{
	oglShaderObject vert(GL_VERTEX_SHADER_ARB);   
	oglShaderObject frag(GL_FRAGMENT_SHADER_ARB); 
	
	GLhandleARB prog=glCreateProgramObjectARB();
	if (!oglToggles['f']) 
	{ /* Compile in vertex and fragment shaders */
		double start_upload=osl::time();
		
		vert.read("vertex.txt");
		frag.read("fragment.txt");
	
		glAttachObjectARB(prog,vert.get());
		glAttachObjectARB(prog,frag.get());
		glLinkProgramARB(prog);
		checkOp(prog,GL_OBJECT_LINK_STATUS_ARB,"link");
		oglCheck("link");
		
		glUseProgramObjectARB(prog);
		oglCheck("use");
		
		/* copy camera location into vertex shader attribute */
		oglVector3d cameraLocation=vp.getEye();
		glUniform4fARB(glGetUniformLocationARB(prog,"cameraLocation"),
			cameraLocation.x,cameraLocation.y,cameraLocation.z,1);
		
		/* Set up texture "sampler" as texture unit 0 */
		glUniform1iARB(glGetUniformLocationARB(prog,"myNoise"),0);
		
		double elapsed_upload=osl::time()-start_upload;
		if (oglToggles['t'])
			printf("Upload: %.2f ms\n",1.0e3*elapsed_upload);
	}
	
	glDisable(GL_BLEND);
	
	drawFragmentObjects(vp);
	
	glUseProgramObjectARB(0);
	glDeleteObjectARB(prog);
}

/******** Program Initialization *********/
int main(int argc, char** argv)
{
	oglOptions i;
	i.name="GL_ARB_shader_language_100 Test";
	i.clearColor=oglVector3d(0.2,0.3,0.4);
	oglSetup(i);
	oglSetupExt();
	if (!GLEW_ARB_fragment_shader) {
		printf("You need programmable shading to run this program!\n");
		exit(1);
	}
	uploadNoise(8);

	oglEventloop(new oglTrackballController(),new fragBench());
	
	return 0;
}
